/************************************************************
Module name:    DIPSLib.c
Notices:        Copyright (c) 1996 Jeffrey Richter
Change Log:     Assorted changes by Dave Jewell for Delphi 
************************************************************/

#include "CmnHdr.H" 
#include <Windows.H>
#include <WindowsX.H>
#include <CommCtrl.H>
#include "Resource.H"

HINSTANCE g_hinstDll = NULL;
HHOOK hHook = NULL;             // gets copied from the app instance to the shell instance
static const TCHAR g_szRegSubKey[] = __TEXT ("Software\\Richter\\Desktop Item Position Saver");

void SaveListViewItemPositions(HWND hwndLV) 
{
   int nItem, nMaxItems = ListView_GetItemCount(hwndLV);
   HKEY hkey;
   DWORD dwDisposition;

   // When saving new positions, delete the old position 
   // information that is currently in the registry.
   RegDeleteKey (HKEY_CURRENT_USER, g_szRegSubKey);

   // Create the registry key to hold the info
   RegCreateKeyEx (HKEY_CURRENT_USER, g_szRegSubKey, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL, &hkey, &dwDisposition);

   for (nItem = 0; nItem < nMaxItems; nItem++) 
   {
      TCHAR szName[_MAX_PATH];
      POINT pt;

      // Get the name and position of a listview item.
      ListView_GetItemText (hwndLV, nItem, 0, szName, _MAX_PATH);
      ListView_GetItemPosition (hwndLV, nItem, &pt);

      // Save the name and position in the registry.
      RegSetValueEx (hkey, szName, 0, REG_BINARY, (PBYTE) &pt, sizeof(pt));
   }

   RegCloseKey(hkey);
}

//--------------------------------------------------------------------------

void RestoreListViewItemPositions (HWND hwndLV) 
{
   HKEY hkey;
   LONG l = RegOpenKeyEx(HKEY_CURRENT_USER, g_szRegSubKey, 0, KEY_QUERY_VALUE, &hkey);
   if (l == ERROR_SUCCESS) {
      int nIndex;

      // If the listview has AutoArrange on, temporarily turn it off.
      DWORD dwStyle = GetWindowStyle (hwndLV);
      if (dwStyle & LVS_AUTOARRANGE) SetWindowLong (hwndLV, GWL_STYLE, dwStyle & ~LVS_AUTOARRANGE);

      l = NO_ERROR;
      for (nIndex = 0; l != ERROR_NO_MORE_ITEMS; nIndex++) {
         TCHAR szName[_MAX_PATH];
         POINT pt;
         LV_FINDINFO lvfi;
         int cbValueName = _MAX_PATH;
         int cbData = sizeof(pt), nItem;
         DWORD dwType;

         // Read a value name and position from the registry.
         l = RegEnumValue(hkey, nIndex, szName, &cbValueName, 
            NULL, &dwType, (PBYTE) &pt, &cbData);

         if (l == ERROR_NO_MORE_ITEMS) continue;

         if ((dwType == REG_BINARY) && (cbData == sizeof(pt))) {
            // The value is something that we recognize; try to find
            // an item in the listview control that matches the name.
            lvfi.flags = LVFI_STRING; 
            lvfi.psz = szName; 
            nItem = ListView_FindItem(hwndLV, -1, &lvfi);
            if (nItem != -1) ListView_SetItemPosition(hwndLV, nItem, pt.x, pt.y);	
         }
      }
      // Turn AutoArrange back on if it was originally on.
      SetWindowLong(hwndLV, GWL_STYLE, dwStyle);
      RegCloseKey(hkey);
   }
}

//--------------------------------------------------------------------------

BOOL WINAPI DllMain (HINSTANCE hinstDll, DWORD fdwReason, LPVOID fImpLoad) 
{
    if (fdwReason == DLL_PROCESS_ATTACH) g_hinstDll = hinstDll;
    return (TRUE);
}

//--------------------------------------------------------------------------

BOOL WINAPI DIPS_DlgProc (HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam) 
{
    switch (uMsg) 
    {
        case WM_APP:
            if (lParam == 1) SaveListViewItemPositions((HWND) wParam); else 
            if (lParam == 0) RestoreListViewItemPositions((HWND) wParam); else 
            if (lParam == 2) ;
            break;

        case WM_CLOSE:
            DestroyWindow (hwnd);
            break;
    }
   
    return (FALSE);
}

//--------------------------------------------------------------------------

LRESULT WINAPI GetMsgProc (int nCode, WPARAM wParam, LPARAM lParam) 
{
   if (hHook == NULL)
   {
      hHook = (HHOOK)((LPMSG)lParam)->lParam;
      // Create the server window to service client requests
      CreateDialog (g_hinstDll, MAKEINTRESOURCE (IDD_DIPS), NULL, DIPS_DlgProc);
      // Tell the original application that server is ready
      PostThreadMessage (((LPMSG)lParam)->wParam, WM_NULL, 0, 0);
   }

   return CallNextHookEx (hHook, nCode, wParam, lParam);
}

//--------------------------------------------------------------------------

__declspec (dllexport) BOOL WINAPI SetDIPSHook (DWORD dwDesktopThread) 
{
   BOOL fOk = FALSE;
   
   if (dwDesktopThread != 0) 
   {
      // Install the hook on the specified thread
      hHook = SetWindowsHookEx (WH_GETMESSAGE, GetMsgProc, g_hinstDll, dwDesktopThread);
      fOk = (hHook != NULL);
      if (fOk) {
         // The hook was installed successfully; force a
         // benign message to the thread's queue so that 
         // the hook function gets called.
         fOk = PostThreadMessage (dwDesktopThread, WM_NULL, GetCurrentThreadId(), (LPARAM) hHook);
      }
   } 
   else 
   {
      fOk = UnhookWindowsHookEx(hHook);
      hHook = NULL;
   }

   return(fOk);
}
